class_name Player extends CharacterBody3D

const FOCUS_SPEED := 30.0
const RUN_SPEED := 80.0
const SPRINT_SPEED := 160.0
const AIR_SPEED := 10.0
const JUMP_FORCE := 4.5
const GROUND_FRICTION := 0.3
const AIR_FRICTION := 0.03

var gravity: Vector3 = (
	ProjectSettings.get_setting("physics/3d/default_gravity")
	* ProjectSettings.get_setting("physics/3d/default_gravity_vector")
)

var selected_interactive: Interactive
var firing := false

@onready var player_hud: PlayerHUD = %PlayerHUD

@onready var camera_pivot: CameraController = %CameraPivot
@onready var spray_muzzle: Marker3D = %SprayMuzzle
@onready var spray_rumbler: Rumbler3D = %SprayRumbler
@onready var interact_ray: RayCast3D = %InteractRay


func get_speed() -> float:
	if is_on_floor():
		if firing:
			return FOCUS_SPEED
		if Input.is_action_pressed("sprint"):
			return SPRINT_SPEED
		return RUN_SPEED
	return AIR_SPEED


func get_friction() -> float:
	if is_on_floor():
		return GROUND_FRICTION
	return AIR_FRICTION


func get_spray() -> Spray:
	return spray_muzzle.get_child(0)


func _physics_process(delta: float) -> void:
	# Will be null if no valid interactor is selected.
	var interactive: Interactive = interact_ray.get_collider() as Interactive
	player_hud.select_interactive(interactive)

	if interactive and Input.is_action_just_pressed("interact"):
		interactive.activate()

	if Input.is_action_pressed("fire"):
		get_spray().fire()
		firing = true
		spray_rumbler.intensity = 0.002
	else:
		get_spray().idle()
		firing = false
		spray_rumbler.intensity *= 0.8

	# Gravity
	if not is_on_floor():
		velocity += gravity * delta

	# Input movement
	var input_dir := Input.get_vector("move_left", "move_right", "move_forward", "move_back")
	var direction := (
		(camera_pivot.global_basis * Vector3(input_dir.x, 0, input_dir.y) * Vector3(1, 0, 1))
		. normalized()
	)
	var movement := direction * get_speed() * delta
	velocity.x += movement.x
	velocity.z += movement.z
	if Input.is_action_just_pressed("jump") and is_on_floor():
		velocity.y = JUMP_FORCE

	# Friction
	var friction := get_friction()
	velocity.x = lerpf(velocity.x, 0, friction)
	velocity.z = lerpf(velocity.z, 0, friction)

	move_and_slide()
